/*  smpcmpx4.cpp  by K.Tsuru  */
/*
This program compares SComplex class with C++ complex one.
It has been tested on gcc.
*/
#include "sn.h"

static const char* const title = "A test program for SComplex class 4";


void ShowTitle(const char* func){
	printf("****************** test of %s ********************\n", func);
}
void TestCacos(){
	ShowTitle("Cacos(z)");
	SComplex z, q, err;
	SDouble x;
	const SDouble RSQ2 = RecSqrt(2.0);
	const int N = 10;
	const double C = 0.5;
	double X, c;
	puts("q = Cacos(z)");
	printf("  real(z)    imag(z)    real(q)    imag(q)        cos(q) - z\n");
Timer timer;
timer.Start();
	for(int i=-N; i<= N;i++){
		X = 2 * i / (double)N;
		x = X * RSQ2;
		c = (abs(i) & 1) ? C : -C;
		z = SComplex(x, c*x);
		q = Cacos(z);
		err = Ccos(q) - z;
		printf("%10.6f %10.6f %10.6f %10.6f %13.3e %13.3e\n",
		 doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()),
		  doubleD(err.Real()), doubleD(err.Imag()));
	}
double t = timer.Stop();
printf("CPU time = %10.3f s\n", t);
}
void TestCasin(){
	ShowTitle("Casin(z)");
	SComplex z, q, err;
	SDouble x;
	const SDouble RSQ2 = RecSqrt(2.0);
	const int N = 10;
	const double C = 5.0;
	double X, c;
	puts("q = Casin(z)");
	printf("  real(z)    imag(z)    real(q)    imag(q)        sin(q) - z\n");
Timer timer;
timer.Start();
	for(int i=-N; i<= N;i++){
		X = 2 * i / (double)N;
		x = X * RSQ2;
		c = (abs(i) & 1) ? C : -C;
		z = SComplex(x, c*x);
		q = Casin(z);
		err = Csin(q) - z;
		printf("%10.6f %10.6f %10.6f %10.6f %13.3e %13.3e\n",
		 doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()),
		  doubleD(err.Real()), doubleD(err.Imag()));
	}
double t = timer.Stop();
printf("CPU time = %10.3f s\n", t);
}
void TestCatan(){
	ShowTitle("Catan(z)");
	SComplex z, q, err;
	SDouble x;
	const SDouble RSQ2 = RecSqrt(2.0);
	const int N = 10;
	const double C = 5.0;
	double X, c;
	puts("q = Catan(z)");
	printf("  real(z)    imag(z)    real(q)    imag(q)        tan(q) - z\n");
Timer timer;
timer.Start();
	for(int i=-N; i<= N;i++){
		X = 2 * i / (double)N;
		x = X * RSQ2;
		c = (abs(i) & 1) ? C : -C;
		z = SComplex(x, c*x);
		q = Catan(z);
		err = Ctan(q) - z;
		printf("%10.6f %10.6f %10.6f %10.6f %13.3e %13.3e\n",
		 doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()),
		  doubleD(err.Real()), doubleD(err.Imag()));
	}
double t = timer.Stop();
printf("CPU time = %10.3f s\n", t);
}
void SomeIdentities(){
	ErrorPuts("It confirms some identities.");
	SComplex diff;
	SDouble x(Sqrt(3.0));

	ErrorPuts("asin(x)- pi/2 + i*arcosh(x)= 0 for x > 1");
	diff = Casin( SDtoSC(x) ) - SDtoSC( MPi2() ) + IU * SDtoSC( Acosh(x) );

	diff.Put();
/*
	ErrorPuts("cosh(i*z)-cos(z) = 0");
	diff = Ccosh(iz) - Ccos(z);
	diff.Put();	putchar('\n');

	ErrorPuts("tanh(i*z)-i*tan(z) = 0");
	diff = Ctanh(iz) - IU*Ctan(z);
	diff.Put();
*/	
	puts("\nDid you get values very close to zero ?");
}

int main(){
	RealSize C;
	C.SetEffFig(50);

	ErrorPuts(title);
	SomeIdentities();	Wait();
	TestCacos();	Wait();
	TestCatan();	Wait();
	TestCasin();

	Ending();
}
