/*  BernTest64bit.cpp  by K.Tsuru  */
const char* title = "<<< A sample program for tangent coefficient, Bernouilli and Euler numbers >>>\n";
#include "sn.h"

void TnTableTest(uint N)
{
	SNBlock <SLong> T(N+1);
	Timer timer;
	double cpu_time;

	cerr << "Calculating tangent coefficients by TanCoeffTable(n) function upto " << N << '.' << endl;
	timer.Start();
	TanCoeffTable(T, N);
	cpu_time = timer.Stop();
	for(uint n = 1; n<= N; n++){
		cout << "T[" << n << "]= " << T(n) << endl;
	}
	cout << "CPU time = " << cpu_time << "sec." << endl;
	Wait();
}

void TanCoeffTest(uint N)
{
	SNBlock <SLong> T(N+1);
	Timer timer;
	double cpu_time;
	uint n;

	cout << "Calcuating tangent coefficients by TanCoeff() function upto " << N << '.' << endl;
	timer.Start();
	for(n = 1; n <= N; n++) T[n-1] = TanCoeff(n);
	cpu_time = timer.Stop();

	for(n = 1; n <= N; n++){
		cout << "T[" << n << "]=" << endl;
		cout << T(n-1) << endl;
	}
	cout << "CPU time = " << cpu_time << " sec." << endl;
	Wait();
}

void BnTableTest(uint N)
{
	SNBlock <SFraction> B(N+1);
	Timer timer;
	double cpu_time;

	cout << "Calculating Bernouilli numbers by BernNumTable(n) function upto " << N << '.' << endl;
	timer.Start();
	BernNumTable(B, N);
	cpu_time = timer.Stop();
	for(uint n = 1; n<= N; n++) {
		cout << "B[" << n << "]= " << B[n] << endl;
	}
	cout << "CPU time = " << cpu_time << " sec." << endl;
	Wait();
#if 1  // test of output to a file in "exe" directory.
    ofstream outF;
    const char* const outFName = "BnTable.txt";
    outF.open(outFName, ios::out);
    if( !outF ) {
        cerr << "Cannot open output stream." << endl;
        return;
    }
	for(uint n = 1; n<= N; n++) {
		outF << "B[" << n << "]= " << B[n] << endl;
	}
	outF.close();
	string msg(outFName);
	msg += " has been made.";
	Wait(msg);
#endif
}

void BernNumTest(uint N){
	SNBlock <SFraction> B(N+1);
	Timer timer;
	double cpu_time;
	uint n;

	cout << "Calculating Bernouilli numbers by BernNum(n) function with combL(n) upto " << N << '.'<< endl;
	timer.Start();
	for(n = 1; n <= N; n++) B[n] = BernNum(n);
	cpu_time = timer.Stop();
	for(n = 1; n <= N; n++){
		cout << "B[" << n << "]= " << B[n] << endl;
	}
	cout << "CPU time = " << cpu_time << " sec." << endl;
	Wait();
}

void EulerNumTest(uint N){
	SNBlock <SLong> E(N+1);
	Timer timer;
	double cpu_time;
	uint n;

	cout << "Calculating Euler numbers by EulerNum(n) function upto " << N << '.' << endl;
	timer.Start();
	for(n=0; n<= N; n++) E[n] = EulerNum(n);
	cpu_time = timer.Stop();

	for(n = 0; n<= N; n++){
		cout << "E[" << n << "]= " << E(n) << endl;
	}
	cout << "CPU time = " << cpu_time << " sec." << endl;
	Wait();
}

int main()
{
	uint N;
	SFraction F;
	F.SetFormat( F.BRACKET | F.END_CR);

	#ifdef _WIN64
		cout << " 64bit system" << endl;
	#endif
	cout << "maxSizeOfMemoryBlock =" << maxSizeOfMemoryBlock << " maxDigits =" << maxDigits << " INT_MAX= " << INT_MAX<< endl;
ulong msz = 1 << 31;
cout << "1 << 31 = " << msz << endl;
msz = PowUL(1024, 4);
cout << "1024^4 = " << msz << endl;
	cout << "LONG_MAX= " << LONG_MAX << " ULONG_MAX= " << ULONG_MAX << " SIZE_MAX= " << SIZE_MAX <<" LONG_LONG_MAX =" << LONG_LONG_MAX << endl;
	SNInfo();Wait();

	cerr << title << endl;
	cerr << "How many terms do you want? ";
	N = GetInt();
	if(!N) return -1;
	BnTableTest(N);
	EulerNumTest(N);
	TnTableTest(N);
	TanCoeffTest(N);
	BernNumTest(N);

	Ending();
}
