// CombSpeedTest.cpp
const char* const title = "Speed test program : combination (binomial) number nCk.";

#include "sn.h"

ulong CombFig(ulong m, ulong n) {
	return Stirling(m) - Stirling(n)-Stirling(m-n);
}

SLong combFact(long n, long k){ // factorial version
	SLong num, den;
	num = Fact(n);
	if(n < n - k) 
	if(k == 0) return 1.0; // includes the case n == k

	den = Fact(n-k)*Fact(k);
	return num/den;
}
void SpeedTestComb2(SLong (*pf1)(ulong, ulong) ,const char* fName1, SLong (*pf2)(ulong, ulong),
const char* fName2, const int repeat){
	Timer timer;
	long n, k;
	SLong r1, r2;
	double t1,t2;
	int c;
	cout << " Compare speed between " << fName1 << " and " << fName2 << endl;
	while(1){
	 	cerr << " To calculate combination number nCk input" << endl;
		cerr << " n(=<0 quit)= ";
		n = GetLong();
		if(n <= 0) return;
		cerr << " k = ";	k = GetLong();
		if(n <= nMax_combD) {
			c=repeat;
			timer.Start();
			while(c--) r1 = combD(n,k);
			t1 = timer.Stop();
			cout << repeat << " times repeat ";
			cout << "cpu time of combD = " << t1 << "(sec)" << endl;
		}
		c=repeat;
		timer.Start();
		while(c--) r1 = (*pf1)(n,k);
		t1 = timer.Stop();

		c=repeat;
		timer.Start();
		while(c--) r2 = (*pf2)(n,k);
		t2 = timer.Stop();

		cout << n << "C" << k << "=" << endl << r1 << endl;
		cout << repeat << " times repeat" << endl;
		cout << "cpu time of " << fName1 << " = " << t1 << "(sec) [ " << r1.DFigures() << "figures]" << endl;
		cout << "cpu time of " << fName2 << " = " << t2 << "(sec) [ " << r2.DFigures() << "figures]" << endl;
		r1 -= r2;
		cout << "r1 - r2 = " << r1 << " = 0?"<< endl;
		Wait();
	}
}

/*
recursive call version
*/
ulong BinomialUL(ulong n, ulong k) { //ulong version. When n < 30 does not overflow.
	k = min(n, n - k);
	ulong r;
	if(k == 0) r = 1;
	else {
//	 	r = (Binomial(n-1, k-1)/n)*k + ((Binomial(n-1, k-1)%k)*n)/k;
	 	r = BinomialUL(n-1, k-1);
	 	r *=n;	r/= k;
	}
	return r;
}
void combULTest(int c) {
	ulong m = 31, n;
	do {
		for(n = 1; n < 31; n++) {
		cout << m << "C" << n <<"=" << combUL(m,n) << endl;
		}
	}while(c--);
}
void combLTest(int c) {
	ulong m = 31, n;
	do {
		for(n = 1; n < 31; n++) {
			cout << m << "C" << n <<"=" << combL(m,n) << endl;
		}
	}while(c--);
}
void combDTest(int c) {
	SLong a, b;
	const ulong m = 40;
	ulong n , f;
	cout << "combDTest" << endl;
	do {
		for(n = 1; n < m; n++) {
			a = combD(m, n);
			b = comb(m,n)-a;
			f = CombFig(m,n);
			cout << m << "C" << n <<"=" << a << " diff =" << b << " CombFig=" << f <<endl;
		}
	}while(c--);
}
SLong binom(ulong n, ulong k) {
	k = min(n, n-k);
	SLong r;
	if(k == 0) r = 1;
	else {
	 	r = binom(n-1, k-1);
	 	r *=n;	r/= k;
	}
	return r;
}
void SpeedTestComb(ulong min, ulong max, ulong step, SLong (*pf)(ulong, ulong) ,const char* funcName){
	ulong n, k;
	SLong r;
	Timer timer;
	timer.Start();
	for( n = min; n <= max ; n += step) {
		k = n/2;
		r = (*pf)(n, k);
	}
	double t = timer.Stop();
	cout << funcName << " : CPU time = " << t << "(sec)" << endl;
}
SLong SpeedTestComb1(ulong m, ulong n, SLong (*pf)(ulong, ulong) ,const char* funcName, double *time){
	SLong r;
	Timer timer;
	cout << m << "C" << n << " is calculating now." << endl;
	int c = 5000;
	timer.Start();
	while(c){
		r = (*pf)(m, n); c--;
	}
	double t = timer.Stop();
	cout << funcName << " : CPU time = " << t << "(sec)" << endl;
	*time = t;
	return r;
}
int main(){
	cout << title << endl;
	ulong m, n;
	m =1000000; n = 1000;
	SLong a = comb(m, n);
	a.SetFormat(10,5,1);
	cout << "comb(" << m << ',' << n << ")="<< endl<< a <<endl << "(" << a.DFigures() << " figs)" << endl;

	SpeedTestComb2(combL, "combL", combPF, "combPF",100);

	SLong s;
	double t1, t2;
	for(m=200; m< 800; m += 100) {
		n = m/2;
		SpeedTestComb1(m,n, combL, "combL", &t1);
		SpeedTestComb1(m,n, combPF, "combPF",&t2);
		if(t1 > t2) cout << "combPF";
		else cout << "combL";
		cout << " is faster." << endl;
	}
	Wait();
	Timer timer;
	timer.Start();
	combULTest(1);
	double t = timer.Stop();
	cout << " : CPU time = " << t << "(sec)" << endl;
	Wait();
	combDTest(0);

	timer.Start();
	combLTest(1);
	t = timer.Stop();
	cout << " : CPU time = " << t << "(sec)" << endl;
	Wait();

	cout << comb(1000,200) << endl;
	ulong N = 50000, step = N /10;
	m = N/10;
	SpeedTestComb(m,N,step, comb,"comb");
	SpeedTestComb(m,N,step, combPF,"combPF");
	SpeedTestComb(m,N,step, combL,"combL"); // fast for small m, N

	Ending();
}

