/* DetThValue.cpp by K.Tsuru*/
/*
Test program to determine the threshold value of
" const uint thFigXXX = xxxx;"
in snmath.h
Run on your platform and determine them.
I made on Aug 8,2016
*/

#include "sn.h"
#include "timer.h"
#include "getnum.h"
#include "wait.h"
/*
Compare the speeds of two functions.
*/
struct ForStm{ // for statement parameters
	int init,max,inc;
	void Set(int i, int m, int ic){
		init=i; max = m; inc = ic;
	}
	ForStm(int i=0, int m=0, int ic=0){Set(i,m,ic);}
};
struct Func {
	string fname;
	SDouble (*pf)(const SDouble&); 
	void Set(string &fn,SDouble (*p)(const SDouble&) ){
		fname =fn;	pf = p;
	}
	Func(string &fn,SDouble (*p)(const SDouble&) ) {
		Set(fn,p);
	}
};
//enum{fINIT, fMAX, fSTEP};
void XFuncSpeedCompare(SDouble (*pfunc1)(const SDouble&), const char* fname1, SDouble (*pfunc2)(const SDouble&), 
const char* fname2, double absmax, const ForStm &F, bool positive = false, double min = 0.0)
{
	Timer timer1, timer2;
	double time1, time2;
	int c = 0;
	RealSize C;

	SDouble r, x, R = Sqrt(0.5);
	int fig;
	const int N = 10;
	int i, Nmin = positive ? 1 : -N;
	SDouble max = abs(absmax);
	while(N*R > max) R /= 2;
	if(min > 0) while(R*Nmin < min) R *= 2;
	cout << "Compare speed depending on effective figures " << fname1 <<" and " << fname2 << endl;
	cout << "            CPU time" << endl;
	cout << " ef  |" << fname1 << "  | " << fname2 << " | ratio  | faster function" << endl;
	for(fig = F.init; fig <= F.max; fig += F.inc){
		c = 0;
		C.SetEffFig(fig);
		R = Sqrt(0.5);
 		timer1.Start();
		for(i = Nmin; i <= N; i++) {
			if(!i) continue;
			x = i * R;
			r = (*pfunc1)(x);
			c++;
		}
		time1 = timer1.Stop();

		timer2.Start();
		for(i = Nmin; i <= N; i++) {
			if(!i) continue;
			x = i * R;
			r = (*pfunc2)(x);
			c++;
		}
		time2 = timer2.Stop();

		cout.precision(5);
		cout << r.EffFig() <<"  " << time1 << "  " << time2 <<"  " << (time1/time2) <<"  " << ((time1 < time2) ? fname1 : fname2) << endl;
		C.SetEffFig(0);
	}
}

int main(){
	RealSize C;
	double mx = 8.0;
/*
Test program to determine the threshold value of
" const uint thFigExpx = 1800u;"
in snmath.h
*/
	ForStm F;
	F.Set(1000, 5000, 500);	
	XFuncSpeedCompare(CosS_RN, "CosS_RN", CosBS, "CosBS", mx, F);

	XFuncSpeedCompare(Cos, "Cos", CosDiv, "CosDiv", mx, F);
	XFuncSpeedCompare(CosS_RN, "CosS_RN", CosDiv, "CosDiv", mx, F);
	Wait();
	F.Set(1600, 3000, 200);	
	XFuncSpeedCompare(ExpBS, "ExpBS", ExpDS, "ExpDS", mx, F);
	Wait();
/*
Test program to determine the threshold value of
" const uint thFigLogx = 1000u;"
in snmath.h
*/
	F.Set(200, 2000, 200);	
//	FIG[fINIT] = 200; FIG[fMAX]=2000;  FIG[fSTEP]=200;	
	XFuncSpeedCompare(LogE, "LogE", LogAGM, "LogAGM", mx, F, true,0.1);

//	Wait();
	Ending();
}
