/* LogX_Test.cpp by K.Tsuru*/
/*
It compares SDouble functions with double ones.
*/

#include "sn.h"

static const char* const title = "A Speed test program for log(x)";

void ShowTitle(const char* func){
	cout << "****************** Speed test of " << func <<"********************" << endl;
}
void ShowCalcInfo(const SDouble& c, double cpu_time){
	printf("Up to %ldth terms, cpu time = %.3f\n",c.UpToTerm(), cpu_time);
}
bool isSmall(const SDouble x) {
	static const SDouble err = 1.0e-11;
	return ( Dabs(x) < err ); 
}
SDouble XFuncSpeedTest(SDouble (*pfunc)(const SDouble&),const char* fname, const SDouble &x, uint fig) {
	Timer timer;
	RealSize C;
	C.SetEffFig(fig); // fig = 10,000
	SDouble result;
	ErrorPuts(title);
	cout << "x ="; x.Puts(5, 15);
	timer.Start();
	result = (*pfunc)(x);
	double time = timer.Stop();
	cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec) (" << result.DFigures()<<"deciam figures) "<< endl;
	return result;
}
void XFuncTest(SDouble (*pfunc)(const SDouble&), double absmax,
    double (*dfunc)(const double), const char* fname, bool positive=false, double min = 0.0)
{
	Timer timer;
	ShowTitle(fname);
	int c = 0;

	SDouble rs, diff;
	double rd;
	double a = sqrt(2.0);
	cout << "  x     f(x) value       difference from double" << endl;
	rs.SetFormat(0, 7);
	cout.precision(7);	
	const int N = 10;
	int i, Nmin = positive ? 1 : -N;
	double x, max = abs(absmax);
	while(N*a > max) a /= 2;
	if(min > 0) while(a*Nmin < min) a *= 2;
	timer.Start();
	for(i = Nmin; i <= N; i++) {
		if(!i) continue;
		x = i * a;
		rs = (*pfunc)(x);
		rd = (*dfunc)(x);
		c++;
		diff = rd - rs;
		cout << x <<"  "<< rd << "     "<< diff << endl;
		if(!isSmall(diff)){
			cerr << "x = " << x << endl;
			cerr << "error is large" << endl; Wait();
		}
	}
	double time = timer.Stop();
	cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)  " << (time/c) << "sec/once"<< endl;
	cout << "in effective figures = " << rs.DFigures() << endl;
Wait();
}
SDouble ExpSqrt2(){ // 100,000degitd 0.98secALM(93 sec) 100 times faster 
	Timer timer;
	timer.Start();
	SDouble r, x = Sqrt(2.0)*0.001;
	r = Exp(x);
	double t = timer.Stop();
	cout << "Exp(Sqrt(2.0))CPU time = " << t << "(sec)" << endl;
	cout << "in effective figures = " << r.DFigures() << endl;
cout << r << endl; 
	cout << "Exp(Sqrt(2.0))CPU time = " << t << "(sec)" << endl;
	cout << "in effective figures = " << r.DFigures() << endl;
	return r;
}
void Compare3Func() {
	SDouble x, rE, rN, rA, dif1, dif2;
	x = DRand(x.MaxSize(),5);
	rE = LogE(x);
	rA = LogAGM(x);
	rN = LogNW(x);
	dif1 = rE - rA;
	dif2 = rA - rN;
	cout << dif1 << "   " << dif2 << "  <-- both are very small?" << endl;
	x.SetFormat(0, 15);
	cout << "x = " << x << endl << "*****************" << endl;
}
int main(){
	RealSize C;
	uint fig;
cerr << "How many effective figures do you want? =";
fig = GetInt();
cerr << "Your fig =" << fig*4 << endl;
	C.SetEffFig(fig); // fig = 10,000
	SDouble x, diff;
//x.ShowMessage(ON);
cerr << " x.MaxSize()=" << x.MaxSize() << endl;
Compare3Func();

	Log10();
/*
	ErrorPuts(title);
	cerr << "x? =";	cin >> x;
	SDouble lnE, lnEN;
	lnEN = LogEN(x);
	lnE = LogE(x);
	diff = lnEN-lnE;
	cout << "Your x = " << x << endl;
	cout << "diff = LogEN(x) - LogE(x) = " << diff << " <-- very small?" << endl;
	Wait();
*/
	double mtx = 1.0, mx = 8.0;
	x = Sqrt(7.5);
	SDouble r1, r2;
//XFuncSpeedTest(LogE,"LogE", x, fig);  	    // 1000       |2,500 | 4000   4500   5,000  10,000(fig)
	XFuncSpeedTest(LogE,"LogE", x, fig);  	    // 0.378(sec)  0.456   0.449  0.487* 0.508*  0.789*
r1= XFuncSpeedTest(LogNW,"LogNW", x, fig); 		// 1.830	   2.12    2.420  3.06   2.91    3.18
r2= XFuncSpeedTest(LogAGM,"LogAGM", x, fig);	// 0.218       0.319   0.397* 0.498  0.703   1.754
cout << "x=" << x << endl;
//x.Puts();
//cout << "r1 =" << r1 << endl;
//r1 = 0.5* r1;
diff = r1 - r2;
cout << "r1 =" << r1 << endl;
cout << "r2 =" << r2 << endl;
cout << "diff =" << diff << endl;
Wait();	

//	XFuncTest(LogBSNW, mx, log, "LogBSNW", true); // 0.166 | 0.260 | 0.4413 | 0.871
	XFuncTest(LogE, mx, log, "Log = LogE", true); // 0.0181 | 0.0997 | 0.296 | 0.588 ... fastest
//	XFuncTest(LogE, mx, log, "Log = LogE", true); // 0.0181 | 0.0997 | 0.296 | 0.588 ... fastest
	XFuncTest(LogAGM, mtx, log, "LogAGM", true); // 0.209 | 0.315 | 0.527 | 1.056
	XFuncTest(Log10, mx, log10, "Log10", true); // 0.0188 |0.1005 | 0.2804 | 0.570

	Ending();
}
