/*  NewtonTest.cpp  by K.Tsuru  */
/*
 r = sqrt(2)
  digits   CPU time(sec) r*r-2
 10000000     3.84       1.0e-10000028
 33554432    30.0       -1.0e-33554460(*)  Kondo 6.6(sec)
 33554432    21.9 (Karatsuba OFF) (faster)
 50000000    33.14       1.0e-50000028
			 33.3 (Karatsuba OFF) (about same)
100000000   112.77       0.0
			134.6 (Karatsuba OFF)
(100million)

FFT normalized   64bit int    double
 500 0000         1.48(sec)    1.63
5000 0000        20.61        23.14  
 r = sqrt(Drand()) random number
  digits   CPU time(sec) r*r-2
 10000000     8.24        -1.0e-10000032
 10000000    10.38(K OFF) -1.0e-10000032(slower)
 50000000    78.36         0.0
 50000000    85.91(K OFF) -1.0 e-50000032
100000000   274.18        -1.0e-100000032
(100million)

************** DReciprocalTest() ******************
1.3769 (sec) x = DRand() end
fig = 100000000
c = 1aTail = 4/  EffFig()=13
delta.NetRdxExp() =-3  ef = 52
maxsizebit = 23  L= 23 tableSize =2097153
c = 2aTail = 15/  EffFig()=61
delta.NetRdxExp() =-14  ef = 244
c = 3aTail = 57/  EffFig()=253
delta.NetRdxExp() =-56  ef = 1012
c = 4aTail = 225/  EffFig()=1021
delta.NetRdxExp() =-224  ef = 4084
c = 5aTail = 899/  EffFig()=4088
delta.NetRdxExp() =-898  ef = 16352
c = 6aTail = 3595/  EffFig()=16375
delta.NetRdxExp() =-3594  ef = 65500
c = 7aTail = 14379/  EffFig()=65527
delta.NetRdxExp() =-14378  ef = 262108
c = 8aTail = 57516/  EffFig()=262135
delta.NetRdxExp() =-57515  ef = 1048540
c = 9aTail = 230064/  EffFig()=1048567
delta.NetRdxExp() =-230063  ef = 4194268
c = 10aTail = 920256/  EffFig()=4194295
delta.NetRdxExp() =-920255  ef = 16777180
c = 11aTail = 3681023/  EffFig()=16777207
delta.NetRdxExp() =-3681022  ef = 67108828
c = 12aTail = 6816451/  EffFig()=67108855
delta.NetRdxExp() =-6816450  ef = 100000000
c = 13aTail = 6722257/  EffFig()=100000000
delta.NetRdxExp() =-6722256  ef = 100000000
  do not converge.
*/
#include "sn.h"
void DReciprocalTest(uint fig){
	cout << "************** DReciprocalTest() ******************" << endl;
	RealSize C(fig);
	Timer timer;
	timer.Start();
	SDouble x = DRand(), r;
	x.Verify(ON);
cout << "Verify() : " << 	x.Verify() << endl;
	cout << timer.LapTime() << " (sec) x = DRand() end" << endl;
	r = DReciprocal(x); // 1/x
	cout << timer.LapTime() << " (sec) r = 1/x = DReciprocal(x) end" << endl;
	Wait();
	cout << (r*x -1.0) << " <-- very close to 0.0 ?"<< endl;
	cout << "Total time : " << timer.Stop() << " (sec) r*x end" << endl;
	Wait();
}
void SqrtSpeed(const SDouble& x, ulong dgt)
{
//	RealSize C;
//	C.SetEffFig(dgt/4);
	Timer timer;
	double t;
	SDouble r, er;
	r.Verify(OFF);

	if(x.Last() <50u) cout << "****** Calculating Sqrt(" << x << ") in " << dgt << "(digits) ******" << endl;
	else cout << "****** Calculating Sqrt(" << doubleD(x) << "...)";
	timer.Start();
	r = Sqrt(x);
	t = timer.Stop();
	er = x - r*r;
	cout << " CPU time : " << t <<"(sec) ";
	cout <<  "figures = " << r.DFigures() << endl;
	cout << "FFTUsedTimes() = " << r.FFTUsedTimes() << "(times)" << endl;
	cerr << "Do you want to output result ?";
	if (dgt > 100000) cerr << " It maybe need much time on DOS.";
	cerr << "Yes(1)/No(0)=? ";
	if(GetInt()) cout << r << endl;
	cout << er << " <== close to 0? " << endl;
	cout << "============================================"<< endl;
	SNInfo();
	cout << "============================================"<< endl;
}
#define SQRT_2 1
int main()
{
FFTInfo();
Wait();
	DReciprocalTest(10000000);
	Timer timer;
	SDouble f;
	RealSize C;
	ulong max_fig = (ulong)f.MaxSize()*(ulong)DFIGURES;
	int dgt;
cout<< "size of size_t = " << sizeof(size_t) << endl;
	cerr << "Maxmum figures in a decimal system = " << max_fig << "." << endl;
	f.Verify(OFF);
	SLong::KaratsubaMultSwitch(true);
#if SQRT_2
	while(1){
		cerr << "How many digits do you want (when <= 0 quit) = ? ";
		if((dgt = GetInt())<=0 ) break;
		C.SetEffFig(dgt/4);
		cerr << "Evaluating sqrt(2) in " << dgt << " (digits)." << endl;
		SqrtSpeed(2.0, dgt);
		C.SetEffFig(0);
	}
#else
	while(1){
		cerr << "How many digits do you want (when <= 0 quit) = ? ";
		if((dgt = GetInt())<=0 ) break;
		ulong n = (ulong)dgt;
		C.SetEffFig(dgt/4);
		cerr << "Now making " << dgt << " digits random number." << endl;
		timer.Start();
		f = Drand();
		double t = timer.Stop();
		cerr << "CPU time = " << t << endl;
		SqrtSpeed(f, n);
		C.SetEffFig(0);
	}
#endif
	Ending();
}
