/*  smpcmpx1.cpp  by K.Tsuru  */
/*  ScomplexTest1.cpp
This program compares SComplex class with C++ complex one.
It has been tested on gcc.
*/
#include "sn.h"
#include "wait.h"

static const char* const title = "A test program for SComplex class";

void ShowTitle(const char* func){
	cout << "****************** test of " << func << " ********************" << endl;
}
void NegativeLog(){
	SComplex z(-1, 0.0), r;
	r = Clog(z);
	cout << "log(-1.0) =" << r << endl << "<-- imaginary part is pi ?" << endl;
}
void TestCexp(){
	ShowTitle("Cexp");
	SComplex z, q;
	SDouble theta;
	int i, N = 10;
	double dr, di, cf = -M_PI;
	complex <double> dz, dq;
	printf("theta      real(z)   imag(z)   real(sq)    imag(sq)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = 2*i/(double)N;
		z = SComplex(theta, cf*theta);
		dz = complex <double>(doubleD(theta), cf*doubleD(theta));
		dq = exp(dz);
		q = Cexp(z);
		dr = real(dq)-doubleD(q.Real());
		di = imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
void TestClog(){
	ShowTitle("Clog");
	SComplex z, q;
	SDouble theta, pi = Pi();
	int i, N = 10;
	double dr, di, cf = -M_PI;
	complex <double> dz, dq;
	printf("theta/pi   real(z)   imag(z)    real(q)     imag(q)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = 2*i/(double)N;
		z = SComplex(Cos(theta*pi), Sin(-theta*pi));
		z *= cf;
		dz = complex <double>(cos(doubleD(theta*pi)), sin(-doubleD(theta*pi)));
		dz *= cf;
		dq = log(dz);
		q = Clog(z);
		dr = real(dq)-doubleD(q.Real());
		di = imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
void TestCsin(){
	ShowTitle("Csin");
	SComplex z, q;
	SDouble theta, pi = Pi(), r;
	int i, N = 10;
	double dr, di;
	complex <double> dz, dq;
	printf("theta/pi   real(z)   imag(z)   real(sq)    imag(sq)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = i/(double)N;
		z = SComplex(theta*pi, theta*pi);
		dz = complex <double>(doubleD(theta*pi), doubleD(theta*pi));
		dq = sin(dz);
		q = Csin(z);
		dr = real(dq)-doubleD(q.Real());
		di = imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
void TestCcos(){
	ShowTitle("Ccos");
	SComplex z, q;
	SDouble theta, pi = Pi(), r;
	int i, N = 10;
	double dr, di;
	complex <double> dz, dq;
	printf("theta/pi   real(z)   imag(z)   real(sq)    imag(sq)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = i/(double)N;
		z = SComplex(theta*pi, theta*pi);
		dz = complex <double>(doubleD(theta*pi), doubleD(theta*pi));
		dq = cos(dz);
		q = Ccos(z);
		dr = real(dq)-doubleD(q.Real());
		di = imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
void TestCsinh(){
	ShowTitle("Csinh");
	SComplex z, q;
	SDouble theta, pi = Pi(), r;
	int i, N = 10;
	double dr, di;
	complex <double> dz, dq;
	printf("theta/pi   real(z)   imag(z)   real(sq)    imag(sq)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = i/(double)N;
		z = SComplex(theta*pi, theta*pi);
		dz = complex <double>(doubleD(theta*pi), doubleD(theta*pi));
		dq = sinh(dz);
		q = Csinh(z);
		dr = real(dq)-doubleD(q.Real());
		di = imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
void TestCcosh(){
	ShowTitle("Ccosh");
	SComplex z, q;
	SDouble theta, pi = Pi(), r;
	int i, N = 10;
	double dr, di;
	complex <double> dz, dq;
	printf("theta/pi   real(z)   imag(z)   real(sq)    imag(sq)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = i/(double)N;
		z = SComplex(theta*pi, theta*pi);
		dz = complex <double>(doubleD(theta*pi), doubleD(theta*pi));
		dq = cosh(dz);
		q = Ccosh(z);
		dr = real(dq)-doubleD(q.Real());
		di = imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
void TestCsqrt(){
	ShowTitle("Csqrt");
	SComplex z, q;
	SDouble theta, pi = Pi();
	int i, N = 10;
	double dr, di;
	complex <double> dz, dq;
	printf("theta/pi   real(z)   imag(z)   real(sq)    imag(sq)  difference from double\n");
	for(i=-N; i<= N;i++){
		theta = 2*i/(double)N;
		z = SComplex(Cos(theta*pi), Sin(theta*pi));
		dz = complex <double>(cos(doubleD(theta*pi)), sin(doubleD(theta*pi)));
		dq = sqrt(dz);
		q = Csqrt(z);
		dr =real(dq)-doubleD(q.Real());
		di =imag(dq)-doubleD(q.Imag());
		printf("%6.1f %10.6f %10.6f %10.6f %10.6f %10.5e %10.5e\n", doubleD(theta), doubleD(z.Real()), doubleD(z.Imag()), doubleD(q.Real()), doubleD(q.Imag()), fabs(dr), fabs(di));
	}
}
void TestCpolar(){
	ShowTitle("Cpolar");
	SComplex z, q;
	complex <double> diff;
	SDouble r;
	int i, N = 10;
z.SetFormat(z.END_CR | z.BracketFMT);
	double dr = sqrt(2.0), angle, pi = M_PI, difr, difi;
	complex <double> dz;
	printf("r      angle/pi  real(z)   imag(z) difference from double\n");
	for(i=-N; i<= N;i++){
		angle = pi*i/(double)N;
		z = Cpolar(dr, angle);
		dz = polar(dr, angle);
//cout << "z = " << dz << endl;
//cout << "z = " << z << endl;
		difr =real(dz)-doubleD(z.Real());
		difi =imag(dz)-doubleD(z.Imag());
//		diff = complexC(z) - dz;
//cout << "dz - z = " << diff << endl;
		
		printf("%6.5f %6.1f %10.6f %10.6f %10.6f %10.5e\n",dr, (angle/pi), doubleD(z.Real()), doubleD(z.Imag()), difr,difi);
	}
}
void TestArg(){
	ShowTitle("Arg");
	SComplex z;
	SDouble angle, diff;
	int ir,ii, N = 4;
	double a = sqrt(2.0), b = sqrt(3.0), angleD, x, y;
	complex <double> zd;
	cout << "  z = (x, y)      angle    difference from double" << endl;

	for(ir=-N; ir<= N;ir++) {
		for(ii=-N; ii<= N;ii++) {
			x = a*ir;	y = b*ii;
			zd.real(x); zd.imag(y);
			angleD = arg(zd);
			z.Set( zd.real(), zd.imag() );
			angle = Arg(z);
			diff = angle - angleD; 	
			cout << zd <<"  "<< angle <<"  "<< diff << endl;
	}
	}
}
/*
(a+bi)/(c+di)
 (a+bi)(c-di)   ac+bd +(bc-ad)i  a*(c/d)+b + {b*(c/d)-a}i
=------------ = ------------- = -------------------------
  c^2+d^2       z.Norm()             c*(c/d) + d
*/
SComplex CCDiv(const SComplex& a, const SComplex& b){
	SDouble re, im, r = a.Real() * b.Real() + a.Imag() * b.Imag();
	SDouble recNorm_z = DReciprocal( b.Norm() );	// = 1/(c^2 + d^2)
	im = ( a.Imag() * b.Real() - a.Real() * b.Imag() ) * recNorm_z;
	re = r * recNorm_z;
	return SComplex(re, im);
}
void TestDiv(){
	int N =10, i, S = 60;
	SComplex a,b,r1, r2, diff;
	for(i = 1 ;i < N;i++) {
		a.Set(Drand(S), Drand(S));
		b.Set(Drand(S), Drand(S));
		r1 = a/b;
if(b.Real() >= b.Imag()) cout << "Re(a) >= Im(b) "<< endl;
else cout << "Re(a) < Im(b)  "<< endl;
		r2 = CCDiv(a,b);
		diff = r1-r2;
		cout << diff << endl;
	}
}
void TestAbs(){
	int N =10, S= 60, i;
	SComplex a;
	SDouble A, B, diff, R = Drand(S)*Dpow10(-50L);
	cout << "Compare Cabs(a, R*a) and CabsH(a, R*a) for R=" << R << endl;
	cout << "*** difference ***" << endl;
	for(i = 1 ;i < N;i++) {
		a.Set(R*Drand(S), Drand(S));
		A = Cabs(a);	B = CabsH(a);
		diff = A - B;
		cout << diff << endl;
	}
}
void TestConstructor(){
	ErrorPuts("***** Test of constructor *****");
	complex<double>dz(1.5 , 0.58);
	SComplex sc1(dz.real(), dz.imag());
	SComplex sc2(dz);

	cout << "sc1 = "<< sc1 <<"  sc2 = " << sc2 << "  <== same?"<< endl;
	SComplex sc3(1.3);
	SComplex sc4 = 1.3;
	cout << "sc3 = "<< sc3 <<"  sc4 = " << sc4 << "  <== same?"<< endl;
	SDouble x = Sqrt(2.0);
	SComplex sc5(x); // OK
//	SComplex sc6 = x; // NG
	cout << "sc5=" << sc5 << " = (sqrt(2), 0.0) ?"<<endl;

}
int main(){
	TestAbs(); Wait();
	TestDiv(); Wait();
	RealSize C;
	C.SetEffFig(5);
	ErrorPuts(title);
	TestConstructor(); Wait();
	TestArg();	Wait();
	TestCpolar(); Wait();
	NegativeLog();
	TestCcosh();
	Wait();
	TestCsinh();
	Wait();
	TestCcos();
	Wait();
	TestCsin();
	Wait();
	TestCexp();
	Wait();
	TestClog();
	Wait();
	TestCsqrt();
	Wait();
	return 0;
}
