/*  ScomplexTest4.cpp
This program compares SComplex class with C++ complex one.
It has been tested on gcc.

C++11 compiler is necessary below.
please add option parameter
 gcc -D__NO_INLINE__ -std=c++11 ...
for gcc version 4.8.1
*/
#include "sn.h"

void ShowEffFig(){
	SDouble t;	
	cout << "Your effective figures in a decimal system = " << t.EffFig()*4 <<" digits." << endl;
} 

typedef complex<double> dcomplex;

static const char* const title = "The test programs for SComplex class.";

void ShowTitle(const char* func){
	cout << "****************** test of " << func << " ********************" << endl;
}
void isSmall(const SComplex& x) {
	static const SDouble err = 1.0e-11;
	SDouble a(Dabs(Re(x))), b(Dabs(Im(x)));
	if( ( a < err ) && ( b < err ) ) ; // do nothing
	else {
		cerr << "error is large" << endl;	exit(-1);
	}
}
void Cpow_nTest() {
	const char* fname = "Cpow(const SComplex& z, int n)";
	Timer timer;
	ShowTitle(fname);
	SComplex zs, rs, diff;
	dcomplex zd, rd;
	int ir, ii, N = 4;
	double a = sqrt(2.0), b = sqrt(3.0);

	cout << "  z = (x, y)     p       z^p value             relative error" << endl;
	zs.Real().SetFormat(8,8);
	cout.precision(7);
	timer.Start();

	for(ir = -N; ir <= N; ir++) {
		for(ii=-N; ii<= N; ii++) {
			if(ii==0 && ir ==0) continue;
			zd = dcomplex(a*ir, b*ii);
			zs = SComplex( zd.real(), zd.imag() );
			for(int p = -N; p <= N ; p++) {
				if(!p) continue;
				rs = Cpow(zs, p);
				rd = pow(zd, p);
				diff = rd/rs - 1.0;
				cout << zd <<"  "<< p << "  " << rs << "  "<< diff << endl;
				isSmall(diff);
			}
		}
	}
	double time = timer.Stop();
cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)" << endl;
}
void Cpow_DTest() {
	const char* fname = "Cpow(const SComplex& x, const SDouble& y);";
	Timer timer;
	ShowTitle(fname);
	SComplex zs, rs, diff;
	dcomplex zd, rd;
	double pd;
	int ir, ii, N = 4;
	double a = sqrt(2.0), b = sqrt(3.0);

	cout << "  z = (x, y)               p       z^p value             relative error" << endl;
	zs.Real().SetFormat(8,8);
cout.precision(7);	
	timer.Start();

	for(ir = -N; ir <= N; ir++) {
		for(ii=-N; ii<= N; ii++) {
			if(ii==0 && ir ==0) continue;
			zd = dcomplex(a*ir, b*ii);
			zs = SComplex( zd.real(), zd.imag() );
			for(int p = -N; p <= N ; p++) {
				if(!p) continue;
				pd = p*a;
				rs = Cpow(zs, pd);
//				rs = Cpow(zs, (SDouble)pd);
				rd = pow(zd, pd);
				diff = rd/rs - 1.0;
				cout << zd <<"  "<< pd << "  " << rd << "  "<< diff << endl;
				isSmall(diff);
			}
		}
	}
	double time = timer.Stop();
cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)" << endl;
}
void Cpow_DpowCTest() {
	const char* fname = "Cpow(const SDouble& a, const SComplex& p);";
	Timer timer;
	ShowTitle(fname);
	SComplex ps, rs, diff;
	dcomplex pd, rd;
	double  a;
	int ir, ii, N = 4;
	double c = sqrt(2.0), d = sqrt(3.0);

	cout << "  a              p = (x, y)            a^p value             relative error" << endl;
	ps.Real().SetFormat(8,8);
cout.precision(7);	
	timer.Start();

	int m= 0;
	for(ir = -N; ir <= N; ir++) {
		for(ii=-N; ii<= N; ii++) {
			if(ii==0 && ir ==0) continue;
			pd = dcomplex(c*ir, d*ii);
			ps = SComplex( pd.real(), pd.imag() );
			for(int ia = -N; ia <= N ; ia++) {
				if(!ia) continue;
				a = ia * d;
				rs = Cpow(a, ps);
				rd = pow(a, pd);
				diff = rd/rs - 1.0;
				cout << a <<"  "<< ps << "  " << rd << "  "<< diff << endl;
				m++;
				isSmall(diff);
			}
		}
	}
	double time = timer.Stop();
cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)  " << m << "times operation"<< endl;
}
void Cpow_CpowCTest() {
	const char* fname = "Cpow(const SComplex& z, const SComplex& p);";
	Timer timer;
	ShowTitle(fname);
	double c = sqrt(2.0), d = sqrt(3.0);
	SComplex as, ps, rs, diff;
	dcomplex a, ad, pd, rd, C(c, d);
	int ir, ii, N = 4;

	cout << "  a              p = (x, y)            a^p value             relative error" << endl;
	ps.Real().SetFormat(8,8);
cout.precision(7);	
	timer.Start();

	int m= 0;
	for(ir = -N; ir <= N; ir++) {
		for(ii=-N; ii<= N; ii++) {
			if(ii==0 && ir ==0) continue;
			pd = dcomplex(c*ir, d*ii);
			ps = SComplex( pd.real(), pd.imag() );
			for(int ia = -N; ia <= N ; ia++) {
				if(!ia) continue;
				a = dcomplex(ia*c, ia*d);
				rs = Cpow(a, ps);
				rd = pow(a, pd);
				diff = rd/rs - 1.0;
				cout << a <<"  "<< ps << "  " << rd << "  "<< diff << endl;
				m++;
				isSmall(diff);
			}
		}
	}
	double time = timer.Stop();
cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)  " << m << "times operation"<< endl;
}
int main(){
/* C++11 compiler is necessary below.*/
	RealSize C;
	C.SetEffFig(60);
	ErrorPuts(title);
	Cpow_DpowCTest();	Wait();
	Cpow_DTest();	Wait();
	Cpow_nTest();	Wait();
	ShowEffFig();   Ending();
}
