/*
test program of sin cos table
1 << 27 = 134 217 728
 INT_MAX = 2 147 483 647
LDBL_DIG = 18
sizeof(long double) =16
DBL_DIG = 15
sizeof(double) =8
FFT_MAX_SIZE_BITS = 27
table size = 33 554 433
Used memory =536 870 928 = 0.5GB
time = 0.536091
MakeSinTable() end
---------------------------------------
long double ldpi =       3.1415926535 8979323846; // give 21 digits
cout << ldpi << endl; // 3.1415926535 89793 11600 //16 digits agree only. The same as double.
Then "long double" cannot be used FFT multiplication GCC at the present time.

*/
//#define USES_SIN_COS_TABLE 1
//#include "sfft.h"
#include "sn.h"
void OkumuraMakeSinTable(int n, long double sintbl[])
{
	int i, n2, n4, n8;
	long double c, s, dc, ds, t, M_PIL=3.14159265358979323846L;

	n2 = n / 2;  n4 = n / 4;  n8 = n / 8;
	t = sinl(M_PIL / n);
	dc = 2 * t * t;  ds = sinl(M_PIL/n2); //sqrtl(dc * (2 - dc));
	t = 2 * dc;  c = sintbl[n4] = 1;  s = sintbl[0] = 0;
	for (i = 1; i < n8; i++) {
		c -= dc;  dc += t * c;
		s += ds;  ds -= t * s;
		sintbl[i] = s;  sintbl[n4 - i] = c;
	}
	if (n8 != 0) sintbl[n8] = sqrtl(0.5L);
	for (i = 0; i < n4; i++)
		sintbl[n2 - i] = sintbl[i];
	for (i = 0; i < n2; i++)	//for (i = 0; i < n2 + n4; i++)
		sintbl[i + n2] = - sintbl[i];
}
void Okumura() { // 18 decimal in long double
cout << "********** Okumura's method sin table ***************" << endl;
	const int L=15;
	const int n = 1 << L;
	long double *st = new long double[n];
	OkumuraMakeSinTable(n, st);
	cout.precision(21);
	SDouble pi = Pi(), c, s , diff, x;
	s.SetFormat(0, 21, 0, 0);
s = Sin(pi/4.0);
c = Cos(pi/4.0);
cout << "Sin(pi/4)=" << s << endl;
cout << "Cos(pi/4)=" << c << endl;
	int i, p;
	for(i = 1; i< L; i++ ){
		p = 1 << i;
		x = 2*(SDouble)p/(SDouble)n;
cout << x << endl;
		s = Sin(pi*x);
		diff = s - st[p] ;
		cout << "sintbl[" << p << "]=" << st[p] << " Sin(2*pi*p/n)=" << s << " diff = " << diff << endl;
	}
	for(p = 1024; p< 2048; p += 64 ){
		x = 2*(SDouble)p/(SDouble)n;
cout <<" x = " <<  x << endl;
		s = Sin(pi*x);
		diff = s - st[p] ;
		cout << "sintbl[" << p << "]=" << st[p] << " Sin(2*pi*p/n)=" << s << " diff = " << diff << endl;
	}
	delete[] st;
	Wait();
}
/// Tailor series expansion
long double sinTaylorSeries(const long double x){
	int k = 2;
 	long double sum = x, x2 = x*x, term = x, err = 1.0e-25;
 	do {	
 		term *= (-1)*x2/((k+1)*k);
 		sum += term;
 		k += 2;
 	} while (abs(term) > err);
	return sum;
}
long double cosTaylorSeries(const long double x){
	int k = 1;
 	long double sum = 1.0, x2= x*x, term = 1.0, err = 1.0e-25;
 	do {	
 		term *= (-1)*x2/((k+1)*k);
 		sum += term;
 		k += 2;
 	} while (abs(term) > err);
	return sum;
}
void TailorTest(){
	cout << "************ sinTaylorSeries(x) test *******************" << endl;
	const int L=15;
	const int N = 1 << L;
	cout.precision(21);
	long double st, x, ldpi = 3.14159265358979323846L; // 21 digits are given.
	double dpi = 3.14159265358979323846;
cout << "double pi = " << dpi << endl;	// 3.141592653589793116 the same as long double
	cout.precision(21);
	SDouble pi = Pi(), s , X, diff, diffX;
	int i, p;
	cout << pi << endl;
cout <<"ldpi = " << ldpi << endl; // 3.1415926535 89793 11600 --- 16 digits agree
	X.SetLongDouble(ldpi); cout << X << endl; // 3.1415926535 8979311600
diff = pi - X;
cout << diff << endl; // 1.22e-16
	s.SetFormat(10, 10);
	cout << " x " << "|   p   | sinTaylorSeries(x) |  SD-ldouble |  x-SD(X)" << endl;
	for(i = 1; i<= L; i++ ){
		p = 1 << i;
		x = 2*ldpi*p/N;
		st = sinTaylorSeries(x);
		X = 2*pi*p/N;
		s = Sin(X);
		diffX = x - X;
		diff = s - st;
		cout << x << "  | " <<  p << "| " << st << " | " << diff << "  |  " << diffX <<endl;
	}	
	for(i = 1; i<= L; i++ ){
		p = 1 << i;
		x = 2*M_PI*p/N;
		st = cosTaylorSeries(x);
		X = 2*pi*p/N;
		s = Cos(X);
		diff = x - X;
		diff = s - st;
		cout <<"x= " << x << " sinTld[" << p << "]=" << st << " diff = " << diff << " diffX = " << diffX <<endl;
	}
	Wait();
}

void About_long_double(){ // N 2^27*16 = 2,147,483,648 = 2GB
	cout << "INT_MAX = " <<  INT_MAX  << endl; // 2147483647
	cout << "LONG_MAX = " << LONG_MAX  << endl; // 2147483647
	cout << "UINT_MAX = " << UINT_MAX  << endl; // 2147483647
	cout << "LDBL_DIG = " << LDBL_DIG  << endl; // 18
	cout << "sizeof(long double) =" << sizeof(long double)<< endl; // = 16 byte = 128bis
	cout << "DBL_DIG = " << DBL_DIG  << endl;  // 15
	cout << "sizeof(double) =" << sizeof(double)<< endl; // = 8 byte = 64bis
	cout << "sizeof(fftType) =" << sizeof(fftType)<< endl; // = 8 byte = 64bis
}

void TablePrecisionTest(){
	fftType sr_n, cr_n;
	SDouble pi = Pi(), pi_4=pi/4;
	SDouble sSD,cSD;
	sSD.SetFormat(10, 10);
	cout.precision(22);
	long j;
	cout << " j     " << "  p " << "  s  "<< "   c  "<< endl;
	long p= 1<< 20, step = 1 << 10;
	for(j = 0; j <=  p; j += step) {
		sr_n = fftSinR(j, p);  cr_n = fftCosR(j, p);
		sSD = Sin(j*pi_4/p);  cSD = Cos(j*pi_4/p);
		sSD -= doubleLD(sr_n);	cSD -= doubleLD(cr_n);
		cout << j  << "  " << p << "  " << sr_n << "  " << cr_n << "  " << sSD<< "  " << cSD << endl;
	}
	cout << sqrtl(0.5)<< endl;
	Wait();
}
void ldoubleSineCosine() {
	const int L = FFT_MAX_SIZE_BITS;
	
// cr[] and sr[] : table of cos(2*pi/(2^r)) and sin(2*pi/(2^r)) r = 0,...,L
	fftType cr[L+1], sr[L+1];
	cr[0] = 1.0;	cr[1] = -1.0;	cr[2] = 0.0;
	sr[0] = sr[1] = 0.0;			sr[2] = 1.0;
//cout.precision(20);
	fftType pi = 3.14159265358979323846L, x;
	uint m = 4, r;
	SDouble X;
	for(r = 3; r < L; r++){
		x= pi/m;
		X.SetLongDouble(x);
		cr[r] = fftCos(x)-doubleLD(Cos(X));
		sr[r] = fftSin(x)-doubleLD(Sin(X));
		m *= 2;
/*		cr[r+1] = fftSqrt( (1.0 + cr[r])/2.0 );
		cr[r+1] = doubleLD( Sqrt( (1.0 + cr[r])/2.0 ) );
		sr[r+1] = 0.5*sr[r]/cr[r+1];	//sr[r+1] = yr[r]/sqrt(2.0*(1.0 + xr[r]));
*/
	}
	cout.precision(21);
	for(r = 0; r < L; r++){
		cout << "cr[" << r << "]=" << cr[r] << endl;
		cout << "sr[" << r << "]=" << sr[r] << endl;
	}	
}
/*
SDouble a, cD, sD, cDiff, sDiff;
long double b;//, sdiff, cdiff;
m = 1;
	for(r = 0; r < L; r++){
		b = 2*M_PI/m;
		a = 2*M_PI/m;
		cdiff = cos(b)-cosl(b); sdiff = sin(b)-sinl(b); // sin(x) = sinl(x) cos(x) = cosl(x)
//cout << "cos(b)-cosl(b)=" << cdiff <<" sin(b)-sinl(b)=" << sdiff << endl;
		cD = Cos(a);	sD = Sin(a);
//		cD -= doubleD(cos(2*M_PI/m)); sD -= doubleD(sin(2*M_PI/m));
		cDiff = cr[r] -cD;	sDiff = sr[r] -sD;
		cout << cDiff << "  " << sDiff << endl;
		m*=2;
	}	
*/
//int MakeSinTable(int maxsizebit);
// int FFTSineTableSize();
//int FFTSineTableUsedMemory();

//typedef long double fftType;
//fftType fftSinR(long n, long d);
//fftType fftCosR(long n, long d);
fftType fftSin_n(long n);
fftType fftCos_n(long n);
SDouble LdoubleToSD(ldouble ld){
	SDouble res;
	res.SetLongDouble(ld);
	return res;		
}
void testOf_stringstream(){
	stringstream ss;
	ldouble pi;
	string MPI = "-3.14159265358979323846";
	cout <<"original MPI= " << MPI << endl;
	ss << MPI;
	cout << "stringstream ss.str() =" << ss.str() << endl;
	ss >> pi;
	cout << "Convert ss --> long double = " << fixed << setprecision(21) << pi << endl;
	string temp = (string)ss.str();
SDouble Spi = temp.c_str();
cout << "Spi = " << Spi << endl;
cout <<"LdoubleToSD(pi) = " << LdoubleToSD(pi) << endl;

cout <<"******* SDouble to long double **********" << endl;
ss << Spi;
ss >> pi;
cout << pi << endl;
pi = doubleLD(Pi());
cout << "doubleLD(Pi()) = " << pi << endl;
Wait("SD to ld");
}
void testOf_gcc_sinl_cosl() {
	cout.precision(20);
	cout<< "************ testOf_gcc_sinl_cosl() **************" << endl;
	ldouble pi = 3.14159265358979323846L;
	ldouble c = cosl(pi/4.0);
	cout << "pi = " << pi << endl;
	cout << "cosl(pi/4.0) = " << c << endl;

	cout << "--- in SDouble ---" << endl;
	SDouble _Pi = Pi(), _C = Cos(_Pi/4.0);
	_Pi.SetFormat(5,24);
	cout << "_Pi = " << _Pi << endl;
	cout << "Cos(_Pi/4.0) = " << _C << endl;

	Wait("******** testOf_gcc_sinl_cosl() ended ************");
}
int main() {
	cout.precision(21);
	long double ldpi = 3.14159265358979323846L; // give 21 digits
	cout << ldpi << endl; // 3.141592653589793238 51 agrees 19 digit 
	ldpi = 4.0L*atanl(1.0L);
	cout << ldpi << endl; // same above
	Wait();

	TailorTest();
	Okumura();
Timer timer;
double t;
#if USES_SIN_TABLE_FFT	
	timer.Start();
	cout << "******** Making Sine Cosine table now **********" << endl;
	MakeSinTable(FFT_MAX_SIZE_BITS);
	t = timer.Stop();
	cout <<" ********** cpu time -->" << t << "(sec)" << endl;
TablePrecisionTest();
#endif
	About_long_double();

	cout << "******** ldoubleSineCosine() **********" << endl;
	timer.Start();
	ldoubleSineCosine();
	t = timer.Stop();
	cout <<" ********** cpu time -->" << t << "(sec)" << endl;

	testOf_gcc_sinl_cosl();
	testOf_stringstream();
	Ending();
}
