/*  SqrtTest.cpp  by K.Tsuru  */
/*
 r = sqrt(2)
  digits   CPU time(sec) r*r-2
 10000000     3.84       1.0e-10000028
 33554432    30.0       -1.0e-33554460(*)  Kondo 6.6(sec)
 33554432    21.9 (Karatsuba OFF) (faster)
 50000000    33.14       1.0e-50000028
			 33.3 (Karatsuba OFF) (about same)
100000000   112.77       0.0
			134.6 (Karatsuba OFF)
**2017/08/06**** Sqrt(2.0) in 100000000(digits) ****** use SinTabele ******
 CPU time : 53.258(sec) figures = 100000029
FFTUsedTimes() = 118(times)
0.0 <== close to 0?(100million)
****** Calculating Sqrt(2.0) in 100000000(digits) ****** nouse SinTabele *** 
 CPU time : 63.1918(sec) figures = 100000029
FFTUsedTimes() = 118(times)
0.0 <== close to 0?
****** Calculating Sqrt(2.0) in 100000000(digits) ****** use SinTabele ******
in MakeSinTable() t = 0.0788352
 CPU time : 62.6489(sec) figures = 100000029 ... almost the same
FFTUsedTimes() = 118(times)
0.0 <== close to 0?

FFT normalized   64bit int    double
 500 0000         1.48(sec)    1.63
5000 0000        20.61        23.14  
 r = sqrt(Drand()) random number
  digits   CPU time(sec) r*r-2
 10000000     8.24        -1.0e-10000032
 10000000    10.38(K OFF) -1.0e-10000032(slower)
 50000000    78.36         0.0
 50000000    85.91(K OFF) -1.0 e-50000032
100000000   274.18        -1.0e-100000032
(100million)
**2017/08/06**** Sqrt(0.231106....) in 100000000(digits) ****** use SinTabele ******
2nd convergence
100000000 53.844(sec)  0.0
4th convergence
100000000 136.265(sec)  0.0 slower

2017/09/17****** Calculating Sqrt(0.475354...) not use SinTabele ******
CPU time : 135.598(sec) figures = 100000032
FFTUsedTimes() = 267(times)
-2.0e-100000032 <== close to 0?
****** Calculating Sqrt(0.679737...)
in MakeSinTable() t = 0.0793405
 CPU time : 138.681(sec) figures = 100000032 ... nearly the same
FFTUsedTimes() = 267(times)
0.0 <== close to 0?
2017/09/17****** Calculating Sqrt(2.0) in 10 0000 0000(digits) ******
 CPU time : 2772(sec) = 46.2(min) figures = 1000000029
FFTUsedTimes() = 3324(times)
1.0e-1000000028 <== close to 0?
*/
#include "sn.h"
void SqrtSpeed(const SDouble& x, ulong dgt)
{
//	RealSize C;
//	C.SetEffFig(dgt/4);
	Timer timer;
	double t;
	SDouble r, er;
	r.Verify(OFF);

	if(x.Last() <50u) cout << "****** Calculating Sqrt(" << x << ") in " << dgt << "(digits) ******" << endl;
	else cout << "****** Calculating Sqrt(" << doubleD(x) << "...)";
	timer.Start();
	r = Sqrt(x);
	t = timer.Stop();
	er = x - r*r;
	cout << " CPU time : " << t <<"(sec) ";
	cout <<  "figures = " << r.DFigures() << endl;
	cout << "FFTUsedTimes() = " << r.FFTUsedTimes() << "(times)" << endl;
	cout << er << " <== close to 0? " << endl;
	cerr << "Do you want to output result ?";
	if (dgt > 100000) cerr << " It maybe need much time on DOS.";
	cerr << "Yes(1)/No(0)=? ";
	if(GetInt()) cout << r << endl;
	cout << "============================================"<< endl;
	SNInfo();
	cout << "============================================"<< endl;
}
#define SQRT_2 1
int main()
{
	Timer timer;
	SDouble f;
	RealSize C;
	ulong max_fig = (ulong)f.MaxSize()*(ulong)DFIGURES;
	int dgt;
cout<< "size of size_t = " << sizeof(size_t) << endl;
	cerr << "Maxmum figures in a decimal system = " << max_fig << "." << endl;
//	f.Verify(ON);	f.FFTVerify(ON);
	SLong::KaratsubaMultSwitch(true);
	FFTInfo();
#if SQRT_2
	while(1){
		cerr << "How many digits do you want (when <= 0 quit) = ? ";
		if((dgt = GetInt())<=0 ) break;
		C.SetEffFig(dgt/4);
		cerr << "Evaluating sqrt(2) in " << dgt << " (digits)." << endl;
		SqrtSpeed(2.0, dgt);
		C.SetEffFig(0);
	}
#else
	while(1){
		cerr << "How many digits do you want (when <= 0 quit) = ? ";
		if((dgt = GetInt())<=0 ) break;
		ulong n = (ulong)dgt;
		C.SetEffFig(dgt/4);
		cerr << "Now making " << dgt << " digits random number." << endl;
		timer.Start();
		f = Drand();
		double t = timer.Stop();
		cerr << "CPU time = " << t << endl;
		SqrtSpeed(f, n);
		C.SetEffFig(0);
	}
#endif
	Ending();
}
