/* cvspeed.cpp by K.Tsuru*/
const char* title =
"<<<<< SLong <--> SInteger radix conversion test >>>>>";
//#include <alloc.h>
#include "sn.h"
#include "timer.h"
#include "wait.h"
#include "getnum.h"
//#define UsesMultBRadix 1
//#define __ErrorCheck
const int N=10;

Timer timer;
void savefile(const char* fname, const SLong& m){
	FILE* fp;
	int j;
	fp = fopen(fname,"wt");
	if(fp == NULL){
		fprintf(stderr, "Cannot open input file %s.\n",fname);
		exit(1);
	}
	for(j = m.Head(); j >=0; j--){
		fprintf(fp,"%04" fTypeFormat,m[j]);
		if((j%16) == 0) fprintf(fp,"\n");
	}
	fclose(fp);
}
struct cpuData{
	uint n;
	double tbin, tnorm;
};
#ifdef __ErrorCheck
void ErrorCheck(){
	SLong a, rN, rD;
	a = a.ReadNumber("errcon.res");
//a /= 10000UL;
	SInteger b;
	b = ConvToBin(a);
	a.RawPuts();
	b.RawPuts();
	rN = b.NConvToDec();
	rD = b.DConvToDec();
	rN.RawPuts();
	rD.RawPuts();
	rN -= a;
	rN.Puts();
	rD -= a;
	rD.Puts();
	a = rN-rD;
	a.Puts();
	Wait();
}
#endif //__ErrorCheck
void rand2bin(cpuData& data){
	SInteger b = LRand(data.n, BRADIX);
	SLong a = b.ConvToDec(), q, r, d;
        int i;
        
	timer.Start();
        i=N;
        while(i){
                 r = b.NConvToDec();	// normal radix conversion
                 i--;
        }
	data.tnorm = timer.Stop();
        q = a-r;
//a.RawPuts();
//r.RawPuts();
	timer.Start();
        i=N;
	while(i){
                 r = b.DConvToDec();	// divide radix conversion
                 i--;
        }
//	r = b.ConvToDec();
//r.RawPuts();
	data.tbin = timer.Stop();
	d = a-r;
//printf("d = ");	d.RawPuts();
	if(d.Sign(555)){
		printf("q = ");	q.RawPuts();
		savefile("errcon.res", a);
		printf("An error occures. The number of figures = %u\n", data.n); 	Wait();
		r = b.DConvToDec();
	}
}
double DconvSpeed(uint fig){
	SLong a;
	SInteger b = LRand(fig, BRADIX);
	double t;
	timer.Start();
	a = b.DConvToDec();
	t = timer.Stop();
	printf("SInteger b has %u figures. SLong a has %u figures.\n", b.Head()+1u, a.Head()+1u);
	return t;
}
void ConvCompare(uint fig){
	SLong a, b;
	SInteger c = LRand(fig, BRADIX);
	double td, tn, r = 1.0;
        const int M = fig < 1000 ? 2*N : 1;
        int j;
	Timer timer;
	timer.Start();
        j=M;
	while(j){   a = c.DConvToDec(); j--; }
	td = timer.Stop();
        timer.Start();
        j=M;
        while(j){   b = c.NConvToDec(); j--; }
	tn = timer.Stop();
	a -= b;
	a.Put();
	puts(" = 0 ?");
	if(tn != 0.0) r = td/tn;
        if(M>1) printf("%d times ", M);
	printf("CPU time DConvToDec() = %.3f, NConvToDec() = %.3f, speed ratio %.3f\n", td, tn, r);
}
void ConvCompareTest(){
	puts("Comparison between DConvToDec() and NConvToDec().");
	uint fig;
	while(1){
		printf("Type in the number of figures in BRADIX f(f < %u abort when 0) = ", SNManager::SNMaxSize(SNManager::BIN_INT) );
		fig = GetInt();
		if(!fig) break;
		ConvCompare(fig);
		Wait();
	}
}
void DconvSpeedTest(){
	puts("Test on the speed of DConvToDec().");
	uint fig;
	double t;
	while(1){
		printf("Type in the number of figures in BRADIX f(f < %u abort when 0) = ", SNManager::SNMaxSize(SNManager::BIN_INT) );
		fig = GetInt();
		if(!fig) break;
		t = DconvSpeed(fig);
		printf(" figures in BRADIX = %u, CPU time = %8.4f\n", fig, t);
		Wait();
	}
}
void corectData(uint from, uint step, int sz, cpuData data[]){
	int j;
	puts("Correcting data now.");
	for(j = 0; j < sz; j++){
		data[j].n = from+step*j;
		rand2bin(data[j]);
	}
}
// N*(logN)^2
double NlogN2(uint n){
	double r = log(n);
	return n*r*r;
}
int convSpeed(){
	int j, from, step;
	double tb, tn, fu, ratio;
	const int dataSize = 10;
	cpuData data[dataSize];

	puts(title);
	puts("Using random numbers it tests the speed of radix conversion.");
	puts("It checks that two results agree each other or not.");
	puts("It does not show any message when agrees.");
	printf("The value of initial figures and its step of increase.(when 0 abort) = ");
	step = from = GetInt();
	if(step == 0) return 0;

	corectData(from, step, dataSize, data);
	j = dataSize-1;
	tb = data[j].tbin;
	tn = data[j].tnorm;
	fu = NlogN2(data[j].n);

	printf("fft size = %d\n", FFTSize());
        printf("CPU time table of %d times conversion.\n", N);
	puts("figures    A:normal conv.       B:divide conv.      N(logN)^2     ratio(A/B)");
	for(j = 0; j < dataSize; j++){
		printf("%5u  %8.2f(%6.2f)  %12.2f(%6.2f) %10.2f(%6.2f)",\
		data[j].n, data[j].tnorm, (data[j].tnorm/tn), data[j].tbin, (data[j].tbin/tb),\
		NlogN2(data[j].n), (NlogN2(data[j].n)/fu) );
		if(data[j].tbin > 0.01){
			ratio = data[j].tnorm/data[j].tbin;
			printf("%8.2f\n", ratio);
		} else puts("   ------");
	}
	puts("                  (a)                     (b)                (c)");
	puts("Normalized value in (a) increses proportional to N^2 and (b) has nearly the same in (c)?");
	if(data[j-1].n > iNconvDecMaxFig){
		printf("When over %d figures the divide conversion is faster?\n",iNconvDecMaxFig);
	}
	puts("The number of figures is in radix DRADIX(=10,000).");
	Wait();
	return 1;
}
#ifdef UsesMultBRadix
void MultBR(){
	SLong a, B(BRADIX), c, d;
	a = LRand(20);
	a.RawPuts();
	d = a*B;
	a = MultBRadix(a);
	d.RawPuts();	a.RawPuts();
	c = d-a;
	c.Puts();
	Wait();
}
#endif	// UsesMultBRadix
int main(){
	RealSize C;
	int s =1;
	SNInfo();
	Wait();
	DconvSpeedTest();
#ifdef __ErrorCheck
	ErrorCheck();
#endif //__ErrorCheck
	ConvCompareTest();
#ifdef UsesMultBRadix
	MultBR();
#endif	//UsesMultBRadix
	while(s) s = convSpeed();
	Ending();
	return EXIT_SUCCESS;
}
